from svglab.elements.common import (
    Element,
    PairedTag,
    Tag,
    TextElement,
    tag_name,
)
from svglab.elements.names import (
    TAG_NAME_TO_NORMALIZED,
    TAG_NAMES,
    TagName,
)
from svglab.elements.tags import (
    A,
    AltGlyph,
    AltGlyphDef,
    AltGlyphItem,
    Animate,
    AnimateColor,
    AnimateMotion,
    AnimateTransform,
    Circle,
    ClipPath,
    ColorProfile,
    Cursor,
    Defs,
    Desc,
    Ellipse,
    FeBlend,
    FeColorMatrix,
    FeComponentTransfer,
    FeComposite,
    FeConvolveMatrix,
    FeDiffuseLighting,
    FeDisplacementMap,
    FeDistantLight,
    FeFlood,
    FeFuncA,
    FeFuncB,
    FeFuncG,
    FeFuncR,
    FeGaussianBlur,
    FeImage,
    FeMerge,
    FeMergeNode,
    FeMorphology,
    FeOffset,
    FePointLight,
    FeSpecularLighting,
    FeSpotLight,
    FeTile,
    FeTurbulence,
    Filter,
    Font,
    FontFace,
    FontFaceFormat,
    FontFaceName,
    FontFaceSrc,
    FontFaceUri,
    ForeignObject,
    G,
    Glyph,
    GlyphRef,
    Hkern,
    Image,
    Line,
    LinearGradient,
    Marker,
    Mask,
    Metadata,
    MissingGlyph,
    Mpath,
    Path,
    Pattern,
    Polygon,
    Polyline,
    RadialGradient,
    Rect,
    Script,
    Set,
    Stop,
    Style,
    Svg,
    Switch,
    Symbol,
    Text,
    TextPath,
    Title,
    Tref,
    Tspan,
    Use,
    View,
    Vkern,
)
from svglab.elements.text_elements import CData, Comment, RawText


__all__ = [
    "TAG_NAMES",
    "TAG_NAME_TO_NORMALIZED",
    "A",
    "AltGlyph",
    "AltGlyphDef",
    "AltGlyphItem",
    "Animate",
    "AnimateColor",
    "AnimateMotion",
    "AnimateTransform",
    "CData",
    "Circle",
    "ClipPath",
    "ColorProfile",
    "Comment",
    "Cursor",
    "Defs",
    "Desc",
    "Element",
    "Ellipse",
    "FeBlend",
    "FeColorMatrix",
    "FeComponentTransfer",
    "FeComposite",
    "FeConvolveMatrix",
    "FeDiffuseLighting",
    "FeDisplacementMap",
    "FeDistantLight",
    "FeFlood",
    "FeFuncA",
    "FeFuncB",
    "FeFuncG",
    "FeFuncR",
    "FeGaussianBlur",
    "FeImage",
    "FeMerge",
    "FeMergeNode",
    "FeMorphology",
    "FeOffset",
    "FePointLight",
    "FeSpecularLighting",
    "FeSpotLight",
    "FeTile",
    "FeTurbulence",
    "Filter",
    "Font",
    "FontFace",
    "FontFaceFormat",
    "FontFaceName",
    "FontFaceSrc",
    "FontFaceUri",
    "ForeignObject",
    "G",
    "G",
    "Glyph",
    "GlyphRef",
    "Hkern",
    "Image",
    "Line",
    "LinearGradient",
    "Marker",
    "Mask",
    "Metadata",
    "MissingGlyph",
    "Mpath",
    "PairedTag",
    "Path",
    "Pattern",
    "Polygon",
    "Polyline",
    "RadialGradient",
    "RawText",
    "RawText",
    "Rect",
    "Rect",
    "Script",
    "Set",
    "Stop",
    "Style",
    "Svg",
    "Svg",
    "Switch",
    "Symbol",
    "Tag",
    "TagName",
    "Text",
    "TextElement",
    "TextPath",
    "Title",
    "Tref",
    "Tspan",
    "Use",
    "View",
    "Vkern",
    "tag_name",
]
